/* 
 * Junebug Demo 
 * Purpose: Demonstrate cooporative multitasking 
 *     using 1 Junebug LED + AC coupled speaker on RB1
 *
 * File: DJ_coop2.c 
 * Software: BoostC or Microchip C18 compilers
 * Hardware: Junebug (or other debugger + breadboarded circuit)
 *
 * by Daniel Johnson
 * July 2008
 */

#include "multiCompiler.h"   

//
// defines & macros
//

#define TRUE  !0
#define FALSE 0

#define byte  unsigned char 
#define uInt  unsigned int

#define state case
#define KMAX  3 
#define BLINK 0
#define BEEP  1
#define DUR   2

#define TASK_BLOCKED 0xFFFF

#define LED_BIT_HI porta_=0b00000001     
#define LED_BIT_LO porta_=0b00000000     
#define SPEAKER_BIT_HI portb_=0b00000010 
#define SPEAKER_BIT_LO portb_=0b00000000 

// allocate KMAX kTimers
uInt kTimer[KMAX];
byte waitForTimer;

void interrupt(void)
{
  byte k;
  // clear the IF  
  intcon_.TMR0IF = 0; 
  for (k=0;k<KMAX;k++)
  {
    // hold at 0
    if((kTimer[k]) &&              // stop at zero
       (kTimer[k]!=TASK_BLOCKED))  // task is blocked
    {
      kTimer[k]--;
    }
  }
  waitForTimer = FALSE;
}

void taskBlink(void)
{
  static byte seq=0;

  switch (seq)
  {
    state 0:
      LED_BIT_HI;
      seq=1;
      break;
    state 1:
      LED_BIT_LO;
      seq=0;
    }
    // set when to run again
    kTimer[BLINK]=1953;
    return;
}

void taskBeep(void)
{ 
  static byte seq=0;
  switch (seq)
  {
    state 0: // state INIT 
      kTimer[DUR]=750;
      seq=1;
      break;
    state 1: // state ON
      SPEAKER_BIT_HI;
      kTimer[BEEP]=2;  // 1000 Hz
      seq=2;
      break;
    state 2: // state OFF
      SPEAKER_BIT_LO;;
      if(kTimer[DUR])  // another pulse
      { 
        kTimer[BEEP]=2;
        seq=1;
      }
      else  // generate a rest
      {
        kTimer[BEEP]=750;
        seq=0;
      }
    }
}

//
// --- main ---
//
void main (void)
{ 
  byte i;
  
  // setup
  {
    // speed up the clock to 8MHz, 18F1320
    osccon_.IRCF0=1;   
    osccon_.IRCF1=1;
    osccon_.IRCF2=1;
    
    // configure ports
    adcon1_ = 0xFF;       // all digital
                          // Switches on RB0 RB2 and RB5
    //intcon2_.RBPU = 0;  // PORTB weak pullups for switches
    trisb_ = 0xFD;        // speaker on RB1
    lata_ = 0;
    trisa_ = 0xBE;     // RA0 and RA6 ouputs 0b1011 1110; 
    
    // configure Timer0
    t0con_ = 0xD0;  
    intcon_.TMR0IF = 0;   // clear the IF
    intcon_.TMR0IE = 1;   // enable TMR0 overflow interrupt
    t0con__.TMR0ON = 1;   // turn on TMR0
    intcon_.GIE = 1;      // enable global interrupts

    // zero out the counters
    for (i=0; i<KMAX;i++) 
    {
      kTimer[i] = 0;
    }  
  }
  // kTimer[BEEP] = TASK_BLOCKED;  // disable taskBeep
  while(1) // main loop
  {
    if (!kTimer[BLINK])
    {
      taskBlink();
    }
    if (!kTimer[BEEP])
    {      
      taskBeep();
    }
    while(waitForTimer);
    waitForTimer = TRUE;
  }
} 